#ifndef CHESSY_BOARD_H
#define CHESSY_BOARD_H

#include "BitBoard.h"

namespace chessy
{
    class Board;
}


class chessy::Board
{
    Bitboard whitePawns;
    Bitboard whiteRooks;
    Bitboard whiteKnights;
    Bitboard whiteBishops;
    Bitboard whiteQueens;
    Bitboard whiteKing;

    Bitboard blackPawns;
    Bitboard blackRooks;
    Bitboard blackKnights;
    Bitboard blackBishops;
    Bitboard blackQueens;
    Bitboard blackKing;

public:
    Board(void) = default;
    Board(const Board&) = default;
    ~Board(void) = default;

    /*!
     * resets the board to the chess starting position.
     */
    void resetBoard(void);

    /*!
     * \return
     *      a bitboard where every field occupied by a white piece
     *      is 1, every free field 0
     */
    Bitboard getWhites(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by a black piece
     *      is 1, every free field 0
     */
    Bitboard getBlacks(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by any piece
     *      is 1, every free field 0
     */
    Bitboard getOccupied(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by any piece
     *      is 0, every free field 1
     */
    Bitboard getFree(void) const;
};

#endif // CHESSY_BOARD_H
